package probes::FPing;

=head1 NAME

probes::FPing - FPing Probe for SmokePing

=head1 SYNOPSIS

 *** Probes ***
 + FPing
 binary = /usr/sepp/bin/fping

=head1 DESCRIPTION

Integrates FPing as a probe into smokeping. The variable binary must
point to your copy of the FPing program. If it is not installed on
your system yet, you can get it from http://www.fping.com/.

=head1 AUTHOR

Tobias Oetiker <tobi@oetiker.ch>

=cut

use strict;
use base qw(probes::base);
use IPC::Open3;
use Symbol;

use Carp;

sub new($$$)
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(@_);

    croak "ERROR: FPing 'binary' not defined in FPing probe definition"
        unless defined $self->{properties}{binary};

    croak "ERROR: FPing 'binary' does not point to an executable"
        unless -x $self->{properties}{binary};

    return $self;
}

sub ping ($){
    my $self = shift;
    # do NOT call superclass ... the ping method MUST be overwriten
    my %upd;
    my $inh = gensym;
    my $outh = gensym;
    my $errh = gensym;
    my $pid = open3($inh,$outh,$errh, 
                    $self->{properties}{binary}, 
                    '-C', $self->{cfg}{Database}{pings}, '-q',
                    @{$self->addresses});
    $self->{rtts}={};
    while (<$errh>){
        chomp;
        my @times = split /\s+/;
        my $ip = shift @times;
        next unless ':' eq shift @times; #drop the colon

        @times = map {sprintf "%.10e", $_ / 10000} sort {$a <=> $b} grep {$_ ne "-"} @times;
        map { $self->{rtts}{$_} = [@times] } @{$self->{addrlookup}{$ip}} ;
    }
    waitpid $pid,0;
    close $inh;
    close $outh;
    close $errh;
}

1;
